/*
 * project_para.h
 *
 *  Created on: Jul 15, 2018
 *      Author: Kasun
 */

#ifndef SRC_PROJECT_PARA_H_
#define SRC_PROJECT_PARA_H_

#include "xparameters.h"
#include "DMA.h"
#include "sleep.h"
#include <stdlib.h>

void init_HiCCE(void);
void init_intan(void);
void HiCCE_DMA_Tranfer(void);
void data_gen_ramp(int datasize);
void HiCCE_DMA_Tranfer_sim(void);

#define DMA_RX_BASEADDR		RX_BUFFER_BASE
#define INTAN_A_RX_BUF_BASE		(DMA_RX_BASEADDR)
#define INTAN_B_RX_BUF_BASE		(DMA_RX_BASEADDR + 0x00020000)
#define INTAN_C_RX_BUF_BASE		(DMA_RX_BASEADDR + 0x00040000)
#define INTAN_D_RX_BUF_BASE		(DMA_RX_BASEADDR + 0x00060000)


#define	data_size_words		MAX_PKT_LEN_WORDS
#define data_size_bytes		data_size_words*4

#define SEND_BUFSIZE (1024)
static char send_buf[SEND_BUFSIZE];
int eTxCnt;

char *gloablStr;
char *globalstr_intan;
#define No_KB_Segs	 64	// only Intan A
//#define No_k_Segs	 512	// all Intans

XTime tStart, tEnd;



#endif /* SRC_PROJECT_PARA_H_ */
